@@ -13,6 +13,6 @@ class By(object):
     DESCRIPTION = "description"
     ROLE = "role"
     STATES = "states"
-    OBJECT_DEPTH = "object_depth"
-    CHILDREN_COUNT = "children_count"
-    INDEX_IN_PARENT = "index_in_parent"
+    OBJECT_DEPTH = "objectdepth"
+    CHILDREN_COUNT = "childrencount"
+    INDEX_IN_PARENT = "indexinparent"
