@@ -1,9 +1,11 @@
+import fnmatch
 import time
 from ctypes.wintypes import HWND
 from typing import Dict, Generator, List, Optional
 import pythoncom
 import win32api
 import win32clipboard
+import win32com.client
 import win32con
 import win32event
 import win32gui
@@ -213,7 +215,8 @@ class Win32Utils(object):
                 self.logger.debug("teardown message pumpup")
                 win32event.SetEvent(self.stop_event)
 
-    def enum_windows(self) -> Dict[HWND, str]:
+    @staticmethod
+    def enum_windows() -> Dict[HWND, str]:
         dict_hwnd = dict()
 
         def get_all_hwnds(hwnd, _):
@@ -228,17 +231,16 @@ class Win32Utils(object):
         return dict_hwnd
 
     def get_hwnd_by_title(self, title: str) -> Optional[HWND]:
-        dict_hwnd = self.enum_windows()
-        try:
-            return list(dict_hwnd.keys())[list(dict_hwnd.values()).index(title)]
-        except ValueError:
-            return None
+        if possible_matches := self.get_hwnds_by_title(title):
+            return possible_matches[0]
+        return None
 
     def get_hwnds_by_title(self, title: str) -> List[HWND]:
         dict_hwnd = self.enum_windows()
-        return [hwnd for hwnd, win_title in dict_hwnd.items() if title == win_title]
+        return [hwnd for hwnd, win_title in dict_hwnd.items() if fnmatch.fnmatch(win_title, title)]
 
-    def get_title_by_hwnd(self, hwnd: HWND) -> str:
+    @staticmethod
+    def get_title_by_hwnd(hwnd: HWND) -> str:
         return win32api.GetWindowText(hwnd)
 
     def wait_hwnd_by_title(self, title: str, timeout: int = TIMEOUT) -> HWND:
@@ -249,51 +251,52 @@ class Win32Utils(object):
                 return hwnd
             error_log = f"no hwnd found by win title =>'{title}'"
             if latest_log != error_log:
-                self.logger.error(error_log)
+                self.logger.debug(error_log)
                 latest_log = error_log
             current = time.time()
             elapsed = round(current - start)
             if elapsed >= timeout:
                 raise TimeoutError(
-                    f"no hwnd found by title '{title}' in '{timeout}'seconds"
+                    f"no hwnd found by title '{title}' in '{timeout}' seconds"
                 )
 
-    def _get_foreground_window(self) -> HWND:
+    @staticmethod
+    def _get_foreground_window() -> HWND:
         return win32gui.GetForegroundWindow()
 
     def _set_window_foreground(self, hwnd: HWND) -> None:
         if hwnd == self._get_foreground_window():
             return
-        win32gui.SetForegroundWindow(hwnd)
-        # TODO: need find another way to set lag here, otherwise foreground will return 0
-        time.sleep(0.1)
-        if hwnd == self._get_foreground_window():
-            return
-        win32gui.ShowWindow(hwnd, win32con.SW_RESTORE)
-        win32gui.ShowWindow(hwnd, win32con.SW_SHOW)
+        win32com.client.Dispatch("WScript.Shell").SendKeys(' ')
         win32gui.SetForegroundWindow(hwnd)
 
-    def _set_window_maximize(self, hwnd: HWND) -> None:
+    @staticmethod
+    def _set_window_maximize(hwnd: HWND) -> None:
         win32gui.ShowWindow(hwnd, win32con.SW_RESTORE)
         win32gui.ShowWindow(hwnd, win32con.SW_MAXIMIZE)
 
-    def _set_window_minimize(self, hwnd: HWND) -> None:
+    @staticmethod
+    def _set_window_minimize(hwnd: HWND) -> None:
         win32gui.ShowWindow(hwnd, win32con.SW_RESTORE)
         win32gui.ShowWindow(hwnd, win32con.SW_MINIMIZE)
 
-    def _set_window_size(self, hwnd: HWND, width: int, height: int) -> None:
+    @staticmethod
+    def _set_window_size(hwnd: HWND, width: int, height: int) -> None:
         left, top, _, _ = win32gui.GetWindowRect(hwnd)
         win32gui.MoveWindow(hwnd, left, top, width, height, True)
 
-    def _set_window_position(self, hwnd: HWND, left: int, top: int) -> None:
+    @staticmethod
+    def _set_window_position(hwnd: HWND, left: int, top: int) -> None:
         _, _, right, bottom = win32gui.GetWindowRect(hwnd)
         win32gui.MoveWindow(hwnd, left, top, left - right, top - bottom, True)
 
-    def _get_window_position(self, hwnd: HWND) -> tuple:
+    @staticmethod
+    def _get_window_position(hwnd: HWND) -> tuple:
         left, top, _, _ = win32gui.GetWindowRect(hwnd)
         return left, top
 
-    def _click_mouse(self, x: int, y: int, hold: int = 0, button: str = "left") -> None:
+    @staticmethod
+    def _click_mouse(x: int, y: int, hold: int = 0, button: str = "left") -> None:
         mouse_down_act = win32con.MOUSEEVENTF_LEFTDOWN if button == "left" else win32con.MOUSEEVENTF_RIGHTDOWN
         mouse_up_act = win32con.MOUSEEVENTF_LEFTUP if button == "left" else win32con.MOUSEEVENTF_RIGHTUP
         win32api.SetCursorPos((x, y))
@@ -302,20 +305,23 @@ class Win32Utils(object):
             time.sleep(hold)
         win32api.mouse_event(mouse_up_act, x, y, 0, 0)
 
-    def _get_clipboard(self) -> str:
+    @staticmethod
+    def _get_clipboard() -> str:
         win32clipboard.OpenClipboard()
         data = win32clipboard.GetClipboardData(win32con.CF_UNICODETEXT)
         win32clipboard.CloseClipboard()
         return data
 
-    def _set_clipboard(self, text: str) -> None:
+    @staticmethod
+    def _set_clipboard(text: str) -> None:
         win32clipboard.OpenClipboard()
         win32clipboard.EmptyClipboard()
         # TODO: error occurs when set clipboard
         win32clipboard.SetClipboardData(win32con.CF_UNICODETEXT, text)
         win32clipboard.CloseClipboard()
 
-    def _empty_clipboard(self) -> None:
+    @staticmethod
+    def _empty_clipboard() -> None:
         win32clipboard.OpenClipboard()
         win32clipboard.EmptyClipboard()
         win32clipboard.CloseClipboard()
@@ -372,7 +378,7 @@ class Win32Utils(object):
     def _send_keys(self, text: str) -> None:
         """Simulate keyboard type for specific text.
 
-        Characters will typed one by one.
+        Characters will be typed one by one.
 
         None-ASCII characters will directly paste to the field(check the security option before).
 
