@@ -1,10 +1,17 @@
 from __future__ import annotations
+
+import os
+import signal
 from ctypes import byref
 from ctypes import CDLL
 from ctypes import c_long
 from ctypes.wintypes import HWND
+from pathlib import Path
+from subprocess import Popen
 from time import time
-from typing import Any, Dict, Tuple, Union
+from typing import Any, Dict, Tuple, Optional
+
+import win32process
 from PIL import ImageGrab
 from pyjab.accessibleinfo import AccessBridgeVersionInfo
 from pyjab.common.actorscheduler import ActorScheduler
@@ -27,40 +34,58 @@ class JABDriver(object):
     """
 
     def __init__(
-        self,
-        title: str = "",
-        bridge_dll: str = "",
-        hwnd: HWND = None,
-        vmid: c_long = None,
-        accessible_context: JOBJECT64 = None,
-        timeout: int = TIMEOUT,
+            self,
+            title: str = "",
+            file_path: Path = None,
+            bridge_dll: str = "",
+            hwnd: HWND = None,
+            vmid: c_long = None,
+            accessible_context: JOBJECT64 = None,
+            timeout: int = TIMEOUT,
     ) -> None:
         """Create a new jab driver.
 
         Args:
-            title (str, optional): Window title of Java appliction need to bind. Defaults to "".
+            title (str, optional): Window title of Java application need to bind. Defaults to "".
+            file_path (Path, optional): File path of the application to launch. Defaults to None.
             bridge_dll (str, optional): WindowsAccessBridge dll file path. Defaults to "".
             hwnd (HWND, optional): HWND of Java Window. Defaults to None.
             vmid (c_long, optional): vmid of Java Window. Defaults to None.
             accessible_context (JOBJECT64, optional): Any Accessible Context Component in Java Window.
             Defaults to None.
-            timeout (int, optional): Default timeout set for JABDriver waitting. Defaults to TIMEOUT.
+            timeout (int, optional): Default timeout set for JABDriver waiting. Defaults to TIMEOUT.
         """
         super(JABDriver, self).__init__()
         self.win32utils = Win32Utils()
+        self.file_path = file_path
+        self._title = title
+        if self.file_path:
+            self.open_application()
         self.serv = Service()
         self.logger = Logger("pyjab")
         self.latest_log = None
         self._bridge_dll = bridge_dll
         self._timeout = timeout
-        self._title = title
         self._hwnd = hwnd
         self._vmid = vmid
+        self._pid = None
         self._accessible_context = accessible_context
         self._bridge = None
         self._root_element = None
         self.init_jab()
-        JABFixedFunc(self._bridge)._fix_bridge_functions()
+        JABFixedFunc(self.bridge)._fix_bridge_functions()
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        os.kill(self.pid, signal.SIGTERM)
+
+    def open_application(self):
+        cmd = " ".join(["javaws", str(self.file_path)]) if self.file_path.suffix == "jnlp" else str(self.file_path)
+        p = Popen(cmd, shell=True)
+        p.wait()
+        return p
 
     @property
     def title(self) -> str:
@@ -78,6 +103,14 @@ class JABDriver(object):
     def hwnd(self, hwnd: HWND) -> None:
         self._hwnd = hwnd
 
+    @property
+    def pid(self) -> int:
+        return self._pid
+
+    @pid.setter
+    def pid(self, pid: int) -> None:
+        self._pid = pid
+
     @property
     def vmid(self) -> c_long:
         return self._vmid
@@ -150,15 +183,16 @@ class JABDriver(object):
         # check if Java Window HWND valid
         if not self._is_java_window(self.hwnd):
             raise RuntimeError(f"HWND:{self.hwnd} is not Java Window, please check!")
+        self.pid = self.get_pid_from_hwnd()
         self.root_element = JABElement(
             bridge=self.bridge,
-            hwnd=HWND(self.hwnd),
-            vmid=c_long(self.vmid),
+            hwnd=self.hwnd,
+            vmid=self.vmid,
             accessible_context=self.accessible_context,
         )
         self.logger.info("init jab success")
 
-    # Gateway funcitons
+    # Gateway functions
     def _is_java_window(self, hwnd: HWND) -> bool:
         """Return the specific window is or not a Java Window
 
@@ -170,7 +204,7 @@ class JABDriver(object):
         """
         return bool(self.bridge.isJavaWindow(hwnd))
 
-    def _get_accessible_context_from_hwnd(self, hwnd: HWND) -> Tuple[JOBJECT64, c_long]:
+    def _get_accessible_context_from_hwnd(self, hwnd: HWND) -> Tuple[JOBJECT64, int]:
         """Gets the AccessibleContext and vmID values for the given window.
 
         Args:
@@ -184,8 +218,11 @@ class JABDriver(object):
         self.bridge.getAccessibleContextFromHWND(
             hwnd, byref(vmid), byref(accessible_context)
         )
-        vmid = vmid.value
-        return accessible_context, vmid
+        return accessible_context, vmid.value
+
+    def get_pid_from_hwnd(self):
+        _, pid = win32process.GetWindowThreadProcessId(self.hwnd)
+        return pid
 
     def get_version_info(self) -> Dict[str, str]:
         """Gets the version information of the instance of Java Access Bridge instance your application is using.
@@ -206,21 +243,21 @@ class JABDriver(object):
             "bridgeWinDLLVersion": info.bridgeWinDLLVersion,
         }
 
-    def get_java_window_hwnd(self, title: str) -> Union[HWND, None]:
+    def get_java_window_hwnd(self, title: str) -> Optional[HWND]:
         """Get Java Window hwnd by title.
 
         Args:
             title (str): Java window title
 
         Returns:
-            Union[HWND, None]: HWND if found Java Window, otherwise return None
+            Optional[HWND]: HWND if found Java Window, otherwise return None
         """
         for hwnd in self.win32utils.get_hwnds_by_title(title=title):
             if self._is_java_window(hwnd):
                 return hwnd
 
     def wait_java_window_by_title(self, title: str, timeout: int = TIMEOUT) -> HWND:
-        """Wait until a Java Window exist in specific seconds.
+        """Wait until a Java Window exists in specific seconds.
 
         Args:
             title (str): The title of specific Java Window need to wait.
@@ -238,7 +275,7 @@ class JABDriver(object):
                 return hwnd
             log_out = f"no java window found by title '{title}'"
             if self.latest_log != log_out:
-                self.logger.warning(log_out)
+                self.logger.debug(log_out)
                 self.latest_log = log_out
             current = time()
             elapsed = round(current - start)
@@ -278,7 +315,7 @@ class JABDriver(object):
 
     def find_element_by_states(self, value: str, visible: bool = False) -> JABElement:
         """
-        Find an JABElement given a states locator.
+        Find an JABElement given a state locator.
         """
         if value == self.root_element.states:
             return self.root_element
@@ -288,10 +325,10 @@ class JABDriver(object):
             )
 
     def find_element_by_object_depth(
-        self, value: int, visible: bool = False
+            self, value: int, visible: bool = False
     ) -> JABElement:
         """
-        Find an JABElement given a object depth locator.
+        Find an JABElement given an object depth locator.
         """
         if value == self.root_element.object_depth:
             return self.root_element
@@ -301,7 +338,7 @@ class JABDriver(object):
             )
 
     def find_element_by_children_count(
-        self, value: int, visible: bool = False
+            self, value: int, visible: bool = False
     ) -> JABElement:
         """
         Find an JABElement given a children count locator.
@@ -314,10 +351,10 @@ class JABDriver(object):
             )
 
     def find_element_by_index_in_parent(
-        self, value: int, visible: bool = False
+            self, value: int, visible: bool = False
     ) -> JABElement:
         """
-        Find an JABElement given a index in parent locator.
+        Find an JABElement given an index in parent locator.
         """
         if value == self.root_element.index_in_parent:
             return self.root_element
@@ -328,12 +365,12 @@ class JABDriver(object):
 
     def find_element_by_xpath(self, value: str, visible: bool = False) -> JABElement:
         """
-        Find an JABElement given a index in parent locator.
+        Find an JABElement given an index in parent locator.
         """
         return self.root_element.find_element_by_xpath(value=value, visible=visible)
 
     def find_element(
-        self, by: str = By.NAME, value: Any = None, visible: bool = False
+            self, by: str = By.NAME, value: Any = None, visible: bool = False
     ) -> JABElement:
         """
         Find an JABElement given a By strategy and locator.
@@ -353,7 +390,7 @@ class JABDriver(object):
         return dict_find[by](value=value, visible=visible)
 
     def find_elements_by_name(
-        self, value: str, visible: bool = False
+            self, value: str, visible: bool = False
     ) -> list[JABElement]:
         """
         Find list of JABElement given a name locator.
@@ -367,7 +404,7 @@ class JABDriver(object):
         return jabelements
 
     def find_elements_by_description(
-        self, value: str, visible: bool = False
+            self, value: str, visible: bool = False
     ) -> list[JABElement]:
         """
         Find list of JABElement given a description locator.
@@ -381,7 +418,7 @@ class JABDriver(object):
         return jabelements
 
     def find_elements_by_role(
-        self, value: str, visible: bool = False
+            self, value: str, visible: bool = False
     ) -> list[JABElement]:
         """
         Find list of JABElement given a role locator.
@@ -395,10 +432,10 @@ class JABDriver(object):
         return jabelements
 
     def find_elements_by_states(
-        self, value: str, visible: bool = False
+            self, value: str, visible: bool = False
     ) -> list[JABElement]:
         """
-        Find list of JABElement given a states locator.
+        Find list of JABElement given a state locator.
         """
         jabelements = []
         if value == self.root_element.states:
@@ -409,10 +446,10 @@ class JABDriver(object):
         return jabelements
 
     def find_elements_by_object_depth(
-        self, value: int, visible: bool = False
+            self, value: int, visible: bool = False
     ) -> list[JABElement]:
         """
-        Find list of JABElement given a object depth locator.
+        Find list of JABElement given an object depth locator.
         """
         jabelements = []
         if value == self.root_element.object_depth:
@@ -425,7 +462,7 @@ class JABDriver(object):
         return jabelements
 
     def find_elements_by_children_count(
-        self, value: int, visible: bool = False
+            self, value: int, visible: bool = False
     ) -> list[JABElement]:
         """
         Find list of JABElement given a children count locator.
@@ -441,10 +478,10 @@ class JABDriver(object):
         return jabelements
 
     def find_elements_by_index_in_parent(
-        self, value: int, visible: bool = False
+            self, value: int, visible: bool = False
     ) -> list[JABElement]:
         """
-        Find list of JABElement given a index in parent locator.
+        Find list of JABElement given an index in parent locator.
         """
         jabelements = []
         if value == self.root_element.index_in_parent:
@@ -457,15 +494,15 @@ class JABDriver(object):
         return jabelements
 
     def find_elements_by_xpath(
-        self, value: str, visible: bool = False
+            self, value: str, visible: bool = False
     ) -> list[JABElement]:
         """
-        Find list of JABElement given a index in parent locator.
+        Find list of JABElement given an index in parent locator.
         """
         return self.root_element.find_elements_by_xpath(value=value, visible=visible)
 
     def find_elements(
-        self, by: str = By.NAME, value: str = None, visible: bool = False
+            self, by: str = By.NAME, value: str = None, visible: bool = False
     ) -> list[JABElement]:
         """
         Find list of JABElement given a By strategy and locator.
@@ -488,16 +525,16 @@ class JABDriver(object):
         """
         Maximizes the current java window that jabdriver is using
         """
-        self.win32utils._set_window_maximize(hwnd=self.root_element.hwnd.value)
+        self.win32utils._set_window_maximize(hwnd=self.root_element.hwnd)
 
     def minimize_window(self):
         """
         Invokes the window manager-specific 'minimize' operation
         """
-        self.win32utils._set_window_minimize(hwnd=self.root_element.hwnd.value)
+        self.win32utils._set_window_minimize(hwnd=self.root_element.hwnd)
 
     def wait_until_element_exist(
-        self, by: str = By.NAME, value: Any = None, timeout: int = TIMEOUT
+            self, by: str = By.NAME, value: Any = None, timeout: int = TIMEOUT
     ) -> JABElement:
         start = time()
         while True:
@@ -541,7 +578,7 @@ class JABDriver(object):
         :Usage:
             driver.get_screenshot_as_base64()
         """
-        self.win32utils._set_window_foreground(hwnd=self.root_element.hwnd.value)
+        self.win32utils._set_window_foreground(hwnd=self.root_element.hwnd)
         bounds = self.root_element.bounds
         x = bounds.get("x")
         y = bounds.get("y")
@@ -570,7 +607,7 @@ class JABDriver(object):
             driver.set_window_size(800,600)
         """
         self.win32utils._set_window_size(
-            hwnd=self.root_element.hwnd.value, width=width, height=height
+            hwnd=self.root_element.hwnd, width=width, height=height
         )
 
     def set_window_position(self, x, y):
@@ -585,7 +622,7 @@ class JABDriver(object):
             driver.set_window_position(0,0)
         """
         self.win32utils._set_window_position(
-            hwnd=self.root_element.hwnd.value, left=x, top=y
+            hwnd=self.root_element.hwnd, left=x, top=y
         )
 
     def get_window_position(self):
@@ -595,4 +632,4 @@ class JABDriver(object):
         :Usage:
             driver.get_window_position()
         """
-        return self.win32utils._get_window_position(hwnd=self.root_element.hwnd.value)
+        return self.win32utils._get_window_position(hwnd=self.root_element.hwnd)
